package com.perfectech.tis.activities;

import android.app.AlertDialog;
import android.app.Dialog;
import android.content.ContentValues;
import android.content.Context;
import android.content.Intent;
import android.content.SharedPreferences;
import android.os.Handler;
import android.os.Bundle;
import android.support.v7.app.AppCompatActivity;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.SimpleAdapter;
import android.widget.TextView;
import android.widget.Toast;

import com.perfectech.tis.DBLogs;
import com.perfectech.tis.DateConvertClass;
import com.perfectech.tis.MessageListTextViewListAdapter;
import com.perfectech.tis.objects.Messages;
import com.perfectech.tis.R;
import com.perfectech.tis.syncClasses.SyncMessages;
import com.perfectech.tis.syncClasses.SyncTasks;
import com.perfectech.tis.TISDBHelper;

import java.io.IOException;
import java.util.ArrayList;


public class UserMessages extends AppCompatActivity {

    private static final String sDBName = "TIS.db";
    static int iVersionVal = 7;
    private static ArrayList<ArrayList<String>> messageArray;
    private String sUserID = "";
    private Handler oHandler = new Handler();

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_user_messages);

        // get the information for the header at the top
        SharedPreferences prefs = getSharedPreferences("TISSharedPrefs",
                Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = prefs.edit();
        String sLoggedInUserCompany = prefs.getString("LoggedInUserCompany", "No Value");
        sUserID = prefs.getString("UserID", "No Value");
        TextView otvUserName = (TextView) findViewById(R.id.tvUserName);
        otvUserName.setText(sLoggedInUserCompany.substring(0,sLoggedInUserCompany.indexOf('-')));
        oHandler.post(timedTask);
     }

    @Override
    public void onResume()
    {
        super.onResume();
        updateUI();
    }

    @Override
    public boolean onCreateOptionsMenu(Menu menu) {
        // Inflate the menu; this adds items to the action bar if it is present.
        getMenuInflater().inflate(R.menu.menu_user_messages, menu);
        return true;
    }

    @Override
    public boolean onOptionsItemSelected(MenuItem item) {
        // Handle action bar item clicks here. The action bar will
        // automatically handle clicks on the Home/Up button, so long
        // as you specify a parent activity in AndroidManifest.xml.
        int id = item.getItemId();

        //noinspection SimplifiableIfStatement
        if (id == R.id.action_create_comm) {
            createMsg();
            return true;
        }
        if (id == R.id.action_check_comms) {
            checkForMessagesFromServer();
            return true;
        }
        if (id == R.id.action_icon_meanings) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID", "IconMeanings");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_help) {
            Intent i = new Intent(getApplicationContext(), HelpActivity.class);
            i.putExtra("ScreenID","UserMessages");
            startActivity(i);
            return true;
        }
        if (id == R.id.action_about) {
            Intent i = new Intent(getApplicationContext(), AboutActivity.class);
            startActivity(i);
            return true;
        }

        return super.onOptionsItemSelected(item);
    }

    private Runnable timedTask = new Runnable() {
        @Override
        public void run() {
            // TODO Auto-generated method stub
            updateUI();
            oHandler.postDelayed(timedTask, 20000);
        }
    };

    private void getMessagesFromDB() {
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSQL = "Select FromUserFullName, MsgStatus, MsgDate, Message, MsgID ";
            sSQL += "from Messages ";
            sSQL += " where MsgStatus <> 'Delete' ";
            sSQL += " and MsgID <> '' ";
            sSQL += " order by MsgDate ";
            Log.d("sSQL", sSQL);
            messageArray = oDB.selectRecordsFromDBList(sSQL, null);
        } catch (Exception ex) {
            DBLogs.insert("Exception: " + ex.toString(),
                    "UserMessage.getMessagesFromDB", getApplicationContext(),
                    sDBName, iVersionVal);
        } finally {
            oDB.close();
        }

        ListView messages = (ListView) this
                .findViewById(R.id.lvCurrentMessages);

        SimpleAdapter sd;
        ArrayList<Messages> allMessages = new ArrayList<Messages>();
        Integer iSize = messageArray.size();
        Log.d("iSize", iSize.toString());
        try {
            SyncMessages.sUserName = sUserID;
            SyncMessages.oContext = getApplicationContext();
            SyncMessages.getMessages();
        } catch (Exception ex) {
            Log.e("SyncMessages Error:", ex.toString());
        }

        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        if (iSize > 0) {
            for (ArrayList<String> sValue : messageArray) {
                Messages msgObj = new Messages();
                msgObj.sFromUserFullName = sValue.get(0);
                msgObj.sMsgStatus = sValue.get(1);
                msgObj.sMessage = sValue.get(3);
                //Message date is GMT in the database so we need to convert to local device date
                msgObj.sMsgDate = oDateConvertClass.GMTToLocal(sValue.get(2));
                msgObj.sMsgID = sValue.get(4);
                allMessages.add(msgObj);
            }

            messages.setAdapter(new MessageListTextViewListAdapter(this, R.layout.messages_row, allMessages));
            messages.setOnItemClickListener(new ListView.OnItemClickListener() {
                public void onItemClick(AdapterView<?> parent, View view,
                                        int position, long id) {
                    try {
                        // get the selected row's ticket number
                        Integer iCounter = 0;
                        for (ArrayList<String> sValue : messageArray) {
                            if (position == iCounter) {
                                Log.d("SelectedMsgTitle", sValue.get(2));
                                Log.d("SelectedMessage", sValue.get(3));
                                Log.d("SelectedMessageID", sValue.get(4));
                                selectMessage(sValue.get(2), sValue.get(3),
                                        sValue.get(4), sValue.get(0));
                            }
                            iCounter++;
                        }
                    } catch (Exception e) {
                        Log.d("Error:", e.toString());
                    }
                }
            });

        } else {
            // No open tickets found
            Log.d("getMessges", "No Messages");
            String[] openTicketsArray = new String[]{"No Messages"};
            messages.setAdapter(new ArrayAdapter<String>(this,
                    android.R.layout.simple_list_item_1, openTicketsArray));
        }

    }
    /********************************************************************************/
	/* Handle the selection of a message */

    /**
     * ****************************************************************************
     */
    private void selectMessage(final String sTitle, final String sMessage,
                               final String sMessageID, final String sFromUser) {
        // open up the dialog box with the message
        final Dialog dialog = new Dialog(UserMessages.this,
                R.style.FullHeightDialog);
        dialog.setContentView(R.layout.dialog_message_with_delete);
        dialog.setCancelable(true);

        TextView tvPrompt = (TextView) dialog.findViewById(R.id.tvDialogHeader);
        //The title is a date so we need to make it local time
        DateConvertClass oDateConvertClass = new DateConvertClass();
        oDateConvertClass.oContext = getApplicationContext();
        String sCurrentDateTime = oDateConvertClass.GMTToLocal(sTitle);
        tvPrompt.setText("From: " + sFromUser + "\r\n" + "Date: " + sCurrentDateTime);

        TextView tvMessage = (TextView) dialog.findViewById(R.id.tvMessage);
        tvMessage.setText(sMessage);

        dialog.show();

        Button btnOK = (Button) dialog.findViewById(R.id.btnOK);
        btnOK.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // Save the data to the db
                dialog.dismiss();
                markMessageRead(sMessageID);
            }
        });

        Button btnDelete = (Button) dialog.findViewById(R.id.btnDelete);
        btnDelete.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // Save the data to the db
                dialog.dismiss();
                markMessageDelete(sMessageID);
            }
        });

        Button btnReply = (Button) dialog.findViewById(R.id.btnReply);
        btnReply.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                // Save the data to the db
                dialog.dismiss();
                msgReply(sMessageID);
            }
        });

        ListView lvMsgList = (ListView) dialog.findViewById(R.id.lvCurrentMessages);
        lvMsgList.invalidate();

    }

    /********************************************************************************/
	/* Handle the reading of a message */

    /**
     * ****************************************************************************
     */
    private void markMessageRead(String sMessageID) {
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(),
                sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            ContentValues oValues = new ContentValues();
            oValues.put("MsgStatus", "Read");
            oDB.updateRecordsInDB("Messages", "MsgID = '"
                    + sMessageID + "'", oValues);
        } catch (Exception ex1) {
            DBLogs.insert("Exception: " + ex1.toString(),
                    "UserMessage.markMessageRead1", getApplicationContext(),
                    sDBName, iVersionVal);
        } finally {
            oDB.close();
        }
        try {
            // queue the message to be sent
            DateConvertClass oDateConvertClass = new DateConvertClass();
            oDateConvertClass.oContext = getApplicationContext();
            String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

            SyncTasks syncTasks = new SyncTasks();
            syncTasks.oContext = getApplicationContext();
            syncTasks.sRecType = "CU";
            syncTasks.sRecID = getID(sMessageID);
            syncTasks.sDateTime = sCurrentDateTime;
            syncTasks.sQueued = "0";
            syncTasks.sDateTimeQueued = "";
            syncTasks.sCompleted = "0";
            syncTasks.sDateTimeCompleted = "";
            syncTasks.write();

            getMessagesFromDB();
        } catch (Exception ex2) {
            DBLogs.insert("Exception: " + ex2.toString(),
                    "UserMessage.markMessageRead2", getApplicationContext(),
                    sDBName, iVersionVal);
        }
    }

    /********************************************************************************/
	/* Handle the deleting of a message */

    /**
     * ****************************************************************************
     */
    private void markMessageDelete(String sMessageID) {
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(),
                sDBName, null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            ContentValues oValues = new ContentValues();
            oValues.put("MsgStatus", "Delete");
            oDB.updateRecordsInDB("Messages", "MsgID = '"
                    + sMessageID + "'", oValues);
        } catch(Exception ex1) {
            DBLogs.insert("Exception: " + ex1.toString(),
                    "UserMessage.markMessageDelete1", getApplicationContext(),
                    sDBName, iVersionVal);
        } finally {
            oDB.close();
        }
        try {
            // queue the message to be sent
            DateConvertClass oDateConvertClass = new DateConvertClass();
            oDateConvertClass.oContext = getApplicationContext();
            String sCurrentDateTime = oDateConvertClass.GMTTimestamp();

            SyncTasks syncTasks = new SyncTasks();
            syncTasks.oContext = getApplicationContext();
            syncTasks.sRecType = "CU";
            syncTasks.sRecID = getID(sMessageID);
            syncTasks.sDateTime = sCurrentDateTime;
            syncTasks.sQueued = "0";
            syncTasks.sDateTimeQueued = "";
            syncTasks.sCompleted = "0";
            syncTasks.sDateTimeCompleted = "";
            syncTasks.write();

            getMessagesFromDB();

        } catch (Exception ex2) {
            DBLogs.insert("Exception: " + ex2.toString(),
                    "UserMessage.markMessageDelete2", getApplicationContext(),
                    sDBName, iVersionVal);
        }
    }

    /********************************************************************************/
	/* Handle the replying to a message */
    /********************************************************************************/
    private void msgReply(String sMessageID) {
        // get previous message from DB for the passed message ID
        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSQL = "Select '\n \n \n  *****'|| FromUserFullName || ' ' || MsgDate || ' ***** \n \n ' ||Message, fromusername, FromUserFullName ";
            sSQL += "from Messages ";
            sSQL += " where MsgID = " + sMessageID;
            Log.d("sSQL", sSQL);
            messageArray = oDB.selectRecordsFromDBList(sSQL, null);
        } catch (Exception ex1) {
            DBLogs.insert("Exception: " + ex1.toString(),
                    "UserMessage.msgReply", getApplicationContext(),
                    sDBName, iVersionVal);
        } finally {
            oDB.close();
        }

        // copy current message to new message
        String sMsgText = "";
        String sFromUserName = "";
        String sFromUserFullName = "";

        if (messageArray.size() > 0) {
            for (ArrayList<String> sValue : messageArray) {
                sMsgText = sValue.get(0);
                sFromUserName = sValue.get(1);
                sFromUserFullName = sValue.get(2);
            }
        }

        // call createMsg
        createMsg(sMsgText, sFromUserName, sFromUserFullName);
    }

    /********************************************************************************/
	/* Handle the deleting of a message */
    /********************************************************************************/
    private void deleteMessage(String sTitle, final String sMessageID) {
        AlertDialog.Builder builder = new AlertDialog.Builder(this);
        /*
        builder.setMessage(R.string.deleteMessageConfirmationText)
                .setCancelable(false)
                .setTitle(R.string.app_name)
                .setIcon(R.drawable.messagebox_warning)
                .setPositiveButton("Yes",
                        new DialogInterface.OnClickListener() {
                            public void onClick(DialogInterface dialog, int id) {
                                try {
                                    OMSDBHelper oDB = new OMSDBHelper(getApplicationContext(),
                                            sDBName, null, 1);
                                    oDB.createDataBase();

                                    oDB.openDataBase();
                                    ContentValues oValues = new ContentValues();
                                    oValues.put("MsgRead", "1");
                                    markMessageDelete(sMessageID);
                                    //oDB.deleteRecordInDB("Messages", "MsgID = '" + sMessageID + "'", null);
                                    oDB.close();
                                } catch (Exception ex) {
                                    Log.d("Error:", ex.toString());
                                }

                                finish();
                                Intent intent = new Intent(getApplicationContext(),
                                        CommunicationCenterActivity.class);
                                intent.putExtra("DriverNo", sDriverNo);
                                startActivity(intent);
                            }
                        })
                .setNegativeButton("No", new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        dialog.cancel();
                    }
                });
        AlertDialog alert = builder.create();
        alert.show();
        */
    }

    /********************************************************************************/
	/* Handle the check messages from server */
    /********************************************************************************/
    private void checkForMessagesFromServer() {
        Toast.makeText(
                getApplicationContext(),
                "Request for communications sent to server.",
                Toast.LENGTH_LONG
        ).show();

        try {
            SyncMessages.sUserName = sUserID;
            SyncMessages.oContext = getApplicationContext();
            SyncMessages.getMessages();
        } catch (Exception ex) {
            Log.e("SyncMessages Error:", ex.toString());
        }
    }

    /********************************************************************************/
	/* Handle the create a message */
    /********************************************************************************/
    private void createMsg(String sMsgText, String sFromUserName, String sFromUserFullName) {
        Intent intent = new Intent(getApplicationContext(),
                CreateCommActivity.class);
        intent.putExtra("InMessageText", sMsgText);
        intent.putExtra("InFromUserName", sFromUserName);
        intent.putExtra("InFromUserFullName", sFromUserFullName);
        startActivity(intent);
    }

    /********************************************************************************/
	/* Handle the create a message */
    /********************************************************************************/
    private void createMsg() {
        Intent intent = new Intent(getApplicationContext(),
                CreateCommActivity.class);
        intent.putExtra("InMessageText", "");
        intent.putExtra("InFromUserName", "");
        intent.putExtra("InFromUserFullName", "");
        startActivity(intent);
    }

    private String getID(String sMsgID) {
        String sRtnVal = "";

        TISDBHelper oDB = new TISDBHelper(getApplicationContext(), sDBName,
                null, iVersionVal);
        try {
            oDB.createDataBase();
            oDB.openDataBase();
            String sSQL = "Select FromUserFullName, MsgStatus, MsgDate, Message, _ID ";
            sSQL += "from Messages ";
            sSQL += " where MsgID = '" + sMsgID + "'";
            messageArray = oDB.selectRecordsFromDBList(sSQL, null);
        } catch (Exception ex) {
            DBLogs.insert("Exception: " + ex.toString(),
                    "UserMessage.msgReply", getApplicationContext(),
                    sDBName, iVersionVal);
        } finally {
            oDB.close();
        }

        for (ArrayList<String> sValue : messageArray) {
            sRtnVal = sValue.get(4);
            Messages msgObj = new Messages();
        }

        return sRtnVal;
    }

    private void updateUI() {
        // get the current messages in the DB and display them in the list view
        getMessagesFromDB();
    }
}
