package com.perfectech.tis.webServices;

import android.content.Context;
import android.util.Log;
import android.location.Address;
import android.location.Geocoder;
import android.location.Criteria;
import android.location.LocationManager;
import android.location.Location;
import android.widget.Toast;

import com.perfectech.tis.DBLogs;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.MalformedInputException;
import java.util.List;

/**
 * Created by jnissley on 7/7/15.
 */
public class SyncFiles {
    public static Context oContext;
    public static File fFilePath;
    public static String sFileType;
    public static String sWebService;
    public static String sFileName;
    private static final int iVersionVal = 7;

    private static final String sDBName = "TIS.db";
    //public static String sUserName;

    /* Upload the signature files to the server.  Make sure to get back an ok before
     * the file is deleted from the device.
     */
    public static Boolean uploadFile() throws Exception {
        boolean bRtnVal = false;
        File fFullFilePath = new File(fFilePath,sFileName);
        HttpURLConnection conn = null;
        DataOutputStream dos = null;

        String boundary = "*****";
        String lineEnd = "\r\n";
        String twoHyphens = "--";
        int bytesRead, bytesAvailable, bufferSize;
        byte[] buffer;
        int maxBufferSize = 1 * 1024 * 1024;
        try {
            //Open the URL to the server
            FileInputStream fileInputStream = new FileInputStream(fFullFilePath);

            int serverResponseCode = 0;
            URL url = new URL(sWebService);
            // Open a HTTP  connection to  the URL
            conn = (HttpURLConnection) url.openConnection();
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use a Cached Copy
            conn.setRequestMethod("POST");
            conn.setRequestProperty("Connection", "Keep-Alive");
            conn.setRequestProperty("ENCTYPE", "multipart/form-data");
            conn.setRequestProperty("Content-Type", "multipart/form-data;boundary=" + boundary);
            //conn.setRequestProperty("Content-Type", "image/png");
            conn.setRequestProperty("uploaded_file", sFileName);
            dos = new DataOutputStream(conn.getOutputStream());
            dos.writeBytes(twoHyphens + boundary + lineEnd);
            dos.writeBytes("Content-Disposition: form-data; name=\"uploaded_file\";filename=\""
                    + sFileName + "\"" + lineEnd);

            dos.writeBytes(lineEnd);
            // create a buffer of  maximum size
            bytesAvailable = fileInputStream.available();

            bufferSize = Math.min(bytesAvailable, maxBufferSize);
            buffer = new byte[bufferSize];

            // read file and write it into form...
            bytesRead = fileInputStream.read(buffer, 0, bufferSize);
            while (bytesRead > 0) {

                dos.write(buffer, 0, bufferSize);
                bytesAvailable = fileInputStream.available();
                bufferSize = Math.min(bytesAvailable, maxBufferSize);
                bytesRead = fileInputStream.read(buffer, 0, bufferSize);

            }

            // send multipart form data necesssary after file data...
            dos.writeBytes(lineEnd);
            dos.writeBytes(twoHyphens + boundary + twoHyphens + lineEnd);

            // Responses from the server (code and message)
            serverResponseCode = conn.getResponseCode();
            String serverResponseMessage = conn.getResponseMessage();

            Log.i("uploadFile", "HTTP Response is : "
                    + serverResponseMessage + ": " + serverResponseCode);

            String oReturnData = "Failed";
            if (serverResponseCode == 200) {
                BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
                String sLine = null;
                StringBuilder sb = new StringBuilder();
                // Read Server Response
                while ((sLine = reader.readLine()) != null) {
                    // Append server response in string
                    sb.append(sLine + "");
                }
                oReturnData = sb.toString();

                if (oReturnData.toLowerCase().equals("success")) {
                    bRtnVal = true; //Automatically false from being defined at the top
                    // Delete the file from the device
                    if (!sFileType.toLowerCase().equals("userfile")) {
                        fFullFilePath.delete();
                    }
                }
            }
            //close the streams //
            fileInputStream.close();
            dos.flush();
            dos.close();
        } catch (MalformedInputException ex) {
            bRtnVal = false;
            DBLogs.insert("MalformedInput Exception " + ex.getMessage(),
                    "SyncFiles.uploadFile",
                    oContext, sDBName, iVersionVal);
        } catch (IOException ioe) {
            bRtnVal = true; //Setting this to true because if file not found we do not want to keep trying
            DBLogs.insert("IOException " + ioe.getMessage(),
                    "SyncFiles.uploadFile",
                    oContext, sDBName, iVersionVal);
        };
        return bRtnVal;
    }

    public static boolean downloadFile () {
        boolean bRtnValue = false;
        int count;
        HttpURLConnection conn = null;
        File fFullFilePath = new File(fFilePath,sFileName);
        try {
            int serverResponseCode = 0;
            URL url = new URL(sWebService);
            // Open a HTTP  connection to  the URL
            conn = (HttpURLConnection) url.openConnection();
            conn.setDoInput(true); // Allow Inputs
            conn.setDoOutput(true); // Allow Outputs
            conn.setUseCaches(false); // Don't use
            // download the file
            InputStream input = new BufferedInputStream(url.openStream(),
                    8192);

            // Output stream
            OutputStream output = new FileOutputStream(fFullFilePath);

            byte data[] = new byte[1024];
            while ((count = input.read(data)) != -1) {
                output.write(data, 0, count);
            }
            // flushing output
            output.flush();

            // closing streams
            output.close();
            input.close();
            bRtnValue = true;
        } catch(IOException ioex) {
            DBLogs.insert("MalformedInput Exception " + ioex.getMessage(),
                    "SyncFiles.downloadFile",
                    oContext, sDBName, iVersionVal);
        }
        return bRtnValue;
    }
    public static Boolean updateStatus(String sJobID, String sUserID, String sPUDelStatus, String sDateTime) {
        Boolean bRtnVal = false;
        String data = "";

        // get current lat/lon to add to the PU/DEL note
        Geocoder geocoder;
        String bestProvider;
        List<Address> user = null;
        double lat = 0;
        double lng = 0;

        LocationManager lm = (LocationManager) oContext.getSystemService(Context.LOCATION_SERVICE);

        Criteria criteria = new Criteria();
        criteria.setAccuracy(Criteria.ACCURACY_FINE);
        bestProvider = lm.getBestProvider(criteria, false);
        Location location = lm.getLastKnownLocation(bestProvider);

        if (location == null){
            Toast.makeText(oContext,"Location Not found",Toast.LENGTH_LONG).show();
        }else{
            geocoder = new Geocoder(oContext);
            try {
                user = geocoder.getFromLocation(location.getLatitude(), location.getLongitude(), 1);
                lat=(double)user.get(0).getLatitude();
                lng=(double)user.get(0).getLongitude();
                System.out.println(" DDD lat: " +lat+",  longitude: "+lng);

            }catch (Exception e) {
                e.printStackTrace();
            }
        }

        String sLat = String.valueOf(lat);
        String sLon = String.valueOf(lng);

        try {
            SyncJson oUpdateStatus = new SyncJson();
            oUpdateStatus.oContext = oContext;
            oUpdateStatus.webserviceName = "UpdPUDelStatusWS.php";
            JSONObject oUpdateStatusObject = new JSONObject();
            oUpdateStatusObject.put("userID", sUserID);
            oUpdateStatusObject.put("jobID", sJobID);
            oUpdateStatusObject.put("PUDelStatus", sPUDelStatus);
            oUpdateStatusObject.put("dateTime",sDateTime);
            oUpdateStatusObject.put("lat", sLat);
            oUpdateStatusObject.put("lon", sLon);
            oUpdateStatus.sendString = oUpdateStatusObject.toString();
            data = oUpdateStatus.SyncData();
        } catch (JSONException ex) {
            DBLogs.insert("Exception:" + ex.toString(),
                    "SyncFiles.updateStatus",
                    oContext, sDBName, iVersionVal);
        }

        if (data.toUpperCase().equals("SUCCESS")) {
            bRtnVal = true;
        }

        return bRtnVal;
    }
}

